#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::control
#
# Weblogic Server control, starts or stops a managed server
#
#  action        = start|running|stop|abort
#  server_type   = admin|managed|ohs_standalone
#  target        = Server|Cluster
#
define wls_install::control (
  Stdlib::Absolutepath   $middleware_home_dir         = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath   $weblogic_home_dir           = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath   $jdk_home_dir                = lookup({name => 'wls_jdk_home_dir'}),
  Optional[
    Stdlib::Absolutepath
  ]                      $wls_domains_dir             = lookup({name =>'wls_domains_dir', default_value => undef}),
  String[1]              $domain_name                 = lookup({name => 'domain_name'}),
  Enum[
    'admin',
    'managed',
    'ohs_standalone'
  ]                      $server_type                 = 'admin',
  Enum[
    'Server',
    'Cluster'
  ]                      $target                      = 'Server',
  String[1]              $server                      = 'AdminServer',
  String[1]              $adminserver_address         = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer                $adminserver_port            = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Boolean                $nodemanager_secure_listener = true,
  Integer                $nodemanager_port            = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  Enum[
    'start',
    'running',
    'stop',
    'abort'
  ]                      $action                      = 'start',
  String[1]              $weblogic_user               = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]              $weblogic_password           = lookup({name => 'domain_wls_password'}),
  Boolean                $jsse_enabled                = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean                $custom_trust                = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]    $trust_keystore_file         = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[String[1]]    $trust_keystore_passphrase   = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  String[1]              $os_user                     = lookup({name => 'wls_os_user'}),
  String[1]              $os_group                    = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath   $download_dir                = lookup({name => 'wls_download_dir'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                      $log_output                  = undef,
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                      $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  if ( $server_type in ['admin', 'ohs_standalone'] ) {
    $ohs_standalone_server = ( $server_type == 'ohs_standalone' )
    wls_adminserver{ "${title}:AdminServer":
      ensure                      => $action,
      server_name                 => $server,
      domain_name                 => $domain_name,
      domain_path                 => $domain_dir,
      os_user                     => $os_user,
      weblogic_home_dir           => $weblogic_home_dir,
      weblogic_user               => $weblogic_user,
      weblogic_password           => $weblogic_password,
      jdk_home_dir                => $jdk_home_dir,
      nodemanager_address         => $adminserver_address,
      nodemanager_port            => $nodemanager_port,
      nodemanager_secure_listener => $nodemanager_secure_listener,
      jsse_enabled                => $jsse_enabled,
      custom_trust                => $custom_trust,
      trust_keystore_file         => $trust_keystore_file,
      trust_keystore_passphrase   => $trust_keystore_passphrase,
      ohs_standalone_server       => $ohs_standalone_server,
    }
  }
  else {
    wls_managedserver{ "${title}:Server":
      ensure              => $action,
      target              => $target,
      server_name         => $server,
      domain_name         => $domain_name,
      os_user             => $os_user,
      weblogic_home_dir   => $weblogic_home_dir,
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password,
      jdk_home_dir        => $jdk_home_dir,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
    }
  }
}
