#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::domain
#
# setup a new weblogic domain
##
define wls_install::domain (
  Wls_install::Versions          $version                               = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath           $weblogic_home_dir                     = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath           $middleware_home_dir                   = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath           $jdk_home_dir                          = lookup({name => 'wls_jdk_home_dir'}),
  Variant[
    Integer[7,7],
    Integer[8,8]
  ]                              $jdk_major_version                     = lookup({name => 'wls_jdk_major_version', default_value => 7}),
  Optional[Stdlib::Absolutepath] $wls_domains_dir                       = lookup({name => 'wls_domains_dir', default_value => undef}),
  Optional[Stdlib::Absolutepath] $wls_apps_dir                          = lookup({name => 'wls_apps_dir', default_value => undef}),
  Enum[
    'standard',
    'ohs_standalone',
    'adf_restricted',
    'osb',
    'osb_soa',
    'osb_soa_bpm',
    'soa',
    'soa_bpm',
    'bam',
    'adf',
    'oim',
    'oud',
    'wc',
    'wc_wcc_bpm'
  ]                              $domain_template                       = lookup({name => 'domain_template', default_value => 'standard'}),
  Boolean                        $bam_enabled                           = true,
  Boolean                        $b2b_enabled                           = false,
  Boolean                        $ess_enabled                           = false,
  Boolean                        $owsm_enabled                          = false,
  String[1]                      $domain_name                           = lookup({name => 'domain_name'}),
  Boolean                        $development_mode                      = true,
  String[1]                      $adminserver_name                      = lookup({name => 'domain_adminserver', default_value => 'AdminServer'}),
  String[1]                      $adminserver_machine_name              = lookup({name => 'domain_adminserver_machine_name', default_value => 'LocalMachine'}),
  String[1]                      $adminserver_address                   = lookup({name => 'domain_adminserver_address', default_value => undef}),
  Integer                        $adminserver_port                      = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Optional[Integer]              $adminserver_ssl_port                  = undef,
  Boolean                        $adminserver_listen_on_all_interfaces  = false,
  Hash                           $java_arguments                        = lookup({name => 'domain_java_arguments', default_value => {}}),
  Optional[String[1]]            $nodemanager_address                   = undef,
  Integer                        $nodemanager_port                      = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  Boolean                        $nodemanager_secure_listener           = true,
  String[1]                      $weblogic_user                         = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]                      $weblogic_password                     = lookup({name => 'domain_wls_password'}),
  Optional[String[1]]            $nodemanager_username                  = undef,
  Optional[String[1]]            $nodemanager_password                  = undef,
  Optional[String[1]]            $domain_password                       = undef,
  Boolean                        $jsse_enabled                          = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean                        $webtier_enabled                       = false,
  String[1]                      $os_user                               = lookup({name => 'wls_os_user'}),
  String[1]                      $os_group                              = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath           $download_dir                          = lookup({name => 'wls_download_dir'}),
  Optional[Stdlib::Absolutepath] $log_dir                               = lookup({name => 'wls_log_dir', default_value => undef}),
  Optional[String[1]]            $repository_database_url               = lookup({name => 'repository_database_url', default_value => undef}),
  Optional[String[1]]            $rcu_database_url                      = undef,
  Optional[Boolean]              $rcu_honor_omf                         = undef,
  Optional[
    Enum[
      'SMALL',
      'MED',
      'LARGE'
    ]
  ]                              $rcu_soa_profile                       = undef,
  String[1]                      $repository_prefix                     = lookup({name => 'repository_prefix', default_value => 'DEV'}),
  String[1]                      $repository_password                   = lookup({name => 'repository_password', default_value => 'Welcome01'}),
  String[1]                      $repository_sys_user                   = 'sys',
  Optional[String[1]]            $repository_sys_password               = undef,
  Boolean                        $custom_trust                          = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]            $trust_keystore_file                   = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[String[1]]            $trust_keystore_passphrase             = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  Boolean                        $custom_identity                       = false,
  Optional[String[1]]            $custom_identity_keystore_filename     = undef,
  Optional[String[1]]            $custom_identity_keystore_passphrase   = undef,
  Optional[String[1]]            $custom_identity_alias                 = undef,
  Optional[String[1]]            $custom_identity_privatekey_passphrase = undef,
  Boolean                        $create_rcu                            = lookup({name => 'create_rcu', default_value => true}),
  Optional[String[1]]            $ohs_standalone_listen_address         = undef,
  Optional[Integer]              $ohs_standalone_listen_port            = undef,
  Optional[Integer]              $ohs_standalone_ssl_listen_port        = undef,
  Optional[String[1]]            $wls_domains_file                      = undef,
  String[1]                      $puppet_os_user                        = 'root',
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                              $log_output                            = undef,
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                              $logoutput                             = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_file ){
    $wls_domains_file_location = $wls_domains_file
  } else {
    $wls_domains_file_location = '/etc/wls_domains.dat'
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  if ( $wls_apps_dir ) {
    $apps_dir =  $wls_apps_dir
  } else {
    $apps_dir = "${middleware_home_dir}/user_projects/applications"
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  unless ( wls_install::domain_exists($domain_dir) ) {
    case $version {
      1036, 1111: {
        $template              = "${weblogic_home_dir}/common/templates/domains/wls.jar"
        $template_ws           = "${weblogic_home_dir}/common/templates/applications/wls_webservice.jar"
        $template_jax_ws       = "${weblogic_home_dir}/common/templates/applications/wls_webservice_jaxws.jar"
        $template_em           = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.em_11_1_1_0_0_template.jar"
        $template_jrf          = "${middleware_home_dir}/oracle_common/common/templates/applications/jrf_template_11.1.1.jar"
        $template_appl_core    = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.applcore.model.stub.11.1.1_template.jar"
        $template_wsmpm        = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.wsmpm_template_11.1.1.jar"
        $template_osb          = "${middleware_home_dir}/Oracle_OSB1/common/templates/applications/wlsb.jar"
        $template_owsm         = "${middleware_home_dir}/Oracle_OSB1/common/templates/applications/wlsb_owsm.jar"
        $template_soa_adapters = "${middleware_home_dir}/Oracle_OSB1/common/templates/applications/oracle.soa.common.adapters_template_11.1.1.jar"
        $template_soa          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.soa_template_11.1.1.jar"
        $template_bpm          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.bpm_template_11.1.1.jar"
        $template_bam          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.bam_template_11.1.1.jar"
        case $domain_template {
          'ohs_standalone': {
            fail("OHS Standalone domain configuration only works with version 12.1.2 or higher. Version ${version} not supported.")
          }
          'standard': {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
          'osb': {
            $extensions_template_file = 'wls_install/domains/extensions/osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_OSB1/common/bin"
          }
          'osb_soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = false
          }
          'osb_soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = true
          }
          'soa', 'soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
          }
          'bam': {
            $extensions_template_file = 'wls_install/domains/extensions/bam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
          }
          'adf', 'adf_restricted': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
          }
          'oim': {
            $extensions_template_file = 'wls_install/domains/extensions/oim_oam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_IDM1/common/bin"
          }
          'oud': {
            $extensions_template_file = 'wls_install/domains/extensions/oud_template.py.erb'
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
          'wc': {
            $extensions_template_file = 'wls_install/domains/extensions/wc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WC1/common/bin"
          }
          'wc_wcc_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/wc_wcc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WCC1/common/bin"
          }
          default: {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
        }
        $rcu_supported            = false
      }
      1112: {
        $template              = "${weblogic_home_dir}/common/templates/domains/wls.jar"
        $template_ws           = "${weblogic_home_dir}/common/templates/applications/wls_webservice.jar"
        $template_em           = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.em_11_1_1_0_0_template.jar"
        $template_jrf          = "${middleware_home_dir}/oracle_common/common/templates/applications/jrf_template_11.1.1.jar"
        $template_appl_core    = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.applcore.model.stub.11.1.1_template.jar"
        $template_wsmpm        = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.wsmpm_template_11.1.1.jar"
        $templateOIM           = "${middleware_home_dir}/Oracle_IDM1/common/templates/applications/oracle.oim_11.1.2.0.0_template.jar"
        $templateOAM           = "${middleware_home_dir}/Oracle_IDM1/common/templates/applications/oracle.oam_ds_11.1.2.0.0_template.jar"
        $templateOUD           = "${middleware_home_dir}/Oracle_OUD1/common/templates/applications/oracle.odsm_11.1.1.5.0_template.jar"
        $template_osb          = "${middleware_home_dir}/Oracle_OSB1/common/templates/applications/wlsb.jar"
        $template_soa_adapters = "${middleware_home_dir}/Oracle_OSB1/common/templates/applications/oracle.soa.common.adapters_template_11.1.1.jar"
        $template_soa          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.soa_template_11.1.1.jar"
        $template_bpm          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.bpm_template_11.1.1.jar"
        $template_bam          = "${middleware_home_dir}/Oracle_SOA1/common/templates/applications/oracle.bam_template_11.1.1.jar"
        case $domain_template {
          'ohs_standalone': {
            fail("OHS Standalone domain configuration only works with version 12.1.2 or higher. Version ${version} not supported.")
          }
          'standard': {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
          'osb': {
            $extensions_template_file = 'wls_install/domains/extensions/osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_OSB1/common/bin"
          }
          'osb_soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = false
          }
          'osb_soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = true
          }
          'soa', 'soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
          }
          'bam': {
            $extensions_template_file = 'wls_install/domains/extensions/bam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
          }
          'adf', 'adf_restricted': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
          }
          'oim': {
            $extensions_template_file = 'wls_install/domains/extensions/oim_oam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_IDM1/common/bin"
          }
          'oud': {
            $extensions_template_file = 'wls_install/domains/extensions/oud_template.py.erb'
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
          'wc': {
            $extensions_template_file = 'wls_install/domains/extensions/wc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WC1/common/bin"
          }
          'wc_wcc_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/wc_wcc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WCC1/common/bin"
          }
          default: {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
          }
        }
        $rcu_supported            = false
      }
      1212: {
        $template           = "${weblogic_home_dir}/common/templates/wls/wls.jar"
        $template_ws        = "${weblogic_home_dir}/common/templates/wls/wls_webservice.jar"
        $template_jax_ws    = "${weblogic_home_dir}/common/templates/wls/wls_webservice_jaxws.jar"
        $template_soap_jms  = "${weblogic_home_dir}/common/templates/wls/wls_webservice_soapjms.jar"
        $template_coherence = "${weblogic_home_dir}/common/templates/wls/wls_coherence.jar"
        $template_em        = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template_12.1.2.jar"
        $template_jrf       = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template_12.1.2.jar"
        $template_appl_core = "${middleware_home_dir}/oracle_common/common/templates/applications/oracle.applcore.model.stub.12.1.3_template.jar"
        $template_wsmpm     = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wsmpm_template_12.1.2.jar"
        $template_em_web_tr = "${middleware_home_dir}/em/common/templates/wls/oracle.em_webtier_template_12.1.2.jar"
        case $domain_template {
          'ohs_standalone': {
            $extensions_template_file = undef
            $wlst_path                = "${middleware_home_dir}/ohs/common/bin"
            $template_file            = 'wls_install/ohs/domain.py.erb'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_standalone_template_12.1.2.jar"
            $rcu_supported            = false
          }
          'standard': {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = false
          }
          'osb': {
            $extensions_template_file = 'wls_install/domains/extensions/osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_OSB1/common/bin"
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'osb_soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'osb_soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'bam': {
            $extensions_template_file = 'wls_install/domains/extensions/bam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_SOA1/common/bin"
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'adf': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $rcu_domain_template      = 'adf'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'adf_restricted': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = false
          }
          'oim': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oim_oam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_IDM1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'oud': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oud_template.py.erb'
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'wc': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WC1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          'wc_wcc_bpm': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_wcc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WCC1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported            = true
          }
          default: {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = undef
            $wlst_path              = "${weblogic_home_dir}/common/bin"
            $template_ohs           = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.2.jar"
            $rcu_supported          = true
          }
        }
      }
      1213: {
        $template           = "${weblogic_home_dir}/common/templates/wls/wls.jar"
        $template_ws        = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-template_12.1.3.jar"
        $template_jax_ws    = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-jaxws-template_12.1.3.jar"
        $template_soap_jms  = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-soapjms-template_12.1.3.jar"
        $template_coherence = "${weblogic_home_dir}/common/templates/wls/wls_coherence_template_12.1.3.jar"
        $template_em        = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template_12.1.3.jar"
        $template_jrf       = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template_12.1.3.jar"
        $template_appl_core = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.applcore.model.stub.1.0.0_template.jar"
        $template_wsmpm     = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wsmpm_template_12.1.3.jar"
        $template_em_web_tr = "${middleware_home_dir}/em/common/templates/wls/oracle.em_webtier_template_12.1.3.jar"
        $template_ess_em    = "${middleware_home_dir}/em/common/templates/wls/oracle.em_ess_template_12.1.3.jar"
        $template_ess       = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.ess.basic_template_12.1.3.jar"
        $template_osb       = "${middleware_home_dir}/osb/common/templates/wls/oracle.osb_template_12.1.3.jar"
        $template_soa       = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa_template_12.1.3.jar"
        $template_bpm       = "${middleware_home_dir}/soa/common/templates/wls/oracle.bpm_template_12.1.3.jar"
        $template_bam       = "${middleware_home_dir}/soa/common/templates/wls/oracle.bam.server_template_12.1.3.jar"
        $template_b2b       = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa.b2b_template_12.1.3.jar"
        $template_health    = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa.healthcare_template_12.1.3.jar"
        case $domain_template {
          'ohs_standalone': {
            $extensions_template_file = undef
            $wlst_path                = "${middleware_home_dir}/ohs/common/bin"
            $template_file            = 'wls_install/ohs/domain.py.erb'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_standalone_template_12.1.3.jar"
            $rcu_supported            = false
          }
          'standard': {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = false
          }
          'osb': {
            $extensions_template_file = 'wls_install/domains/extensions/osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/osb/common/bin"
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'osb_soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/soa/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'osb_soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/soa/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/soa/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/soa/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'bam': {
            $extensions_template_file = 'wls_install/domains/extensions/bam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/soa/common/bin"
            $rcu_domain_template      = 'soa'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'adf': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $rcu_domain_template      = 'adf'
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'adf_restricted': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = false
          }
          'oim': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oim_oam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_IDM1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'oud': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oud_template.py.erb'
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'wc': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WC1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          'wc_wcc_bpm': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_wcc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_WCC1/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
          default: {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template_12.1.3.jar"
            $rcu_supported            = true
          }
        }
      }
      1221, 12211, 12212, 12213: {
        $template           = "${weblogic_home_dir}/common/templates/wls/wls.jar"
        $template_ws        = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-template.jar"
        $template_jax_ws    = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-jaxws-template.jar"
        $template_soap_jms  = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wls-webservice-soapjms-template.jar"
        $template_coherence = "${weblogic_home_dir}/common/templates/wls/wls_coherence_template.jar"
        $template_appl_core = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.applcore.model.stub_template.jar"
        $template_wsmpm     = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.wsmpm_template.jar"
        $template_em_web_tr = "${middleware_home_dir}/em/common/templates/wls/oracle.em_webtier_template.jar"
        $template_ess_em    = "${middleware_home_dir}/em/common/templates/wls/oracle.em_ess_template.jar"
        $template_ess       = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.ess.basic_template.jar"
        $template_osb       = "${middleware_home_dir}/osb/common/templates/wls/oracle.osb_template.jar"
        $template_soa       = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa_template.jar"
        $template_bpm       = "${middleware_home_dir}/soa/common/templates/wls/oracle.bpm_template.jar"
        $template_bam       = "${middleware_home_dir}/soa/common/templates/wls/oracle.bam.server_template.jar"
        $template_b2b       = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa.b2b_template.jar"
        $template_health    = "${middleware_home_dir}/soa/common/templates/wls/oracle.soa.healthcare_template.jar"
        case $domain_template {
          'ohs_standalone': {
            $extensions_template_file = undef
            $wlst_path                = "${middleware_home_dir}/ohs/common/bin"
            $template_file            = 'wls_install/ohs/domain.py.erb'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_standalone_template.jar"
            $rcu_supported            = false
          }
          'standard': {
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = false
          }
          'osb': {
            $extensions_template_file = 'wls_install/domains/extensions/osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'osb_soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'osb_soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_osb_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'soa': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $bpm                      = false
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'soa_bpm': {
            $extensions_template_file = 'wls_install/domains/extensions/soa_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $bpm                      = true
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'bam': {
            $extensions_template_file = 'wls_install/domains/extensions/bam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $rcu_domain_template      = 'soa'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'adf': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $rcu_domain_template      = 'adf'
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'adf_restricted': {
            $extensions_template_file = 'wls_install/domains/extensions/jrf_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_restricted_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_restricted_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_jrf_restricted_template.jar"
            $restricted = true
            $rcu_supported            = false
          }
          'oim': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oim_oam_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/Oracle_IDM1/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'oud': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/oud_template.py.erb'
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'wc': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          'wc_wcc_bpm': {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = 'wls_install/domains/extensions/wc_wcc_template.py.erb'
            $wlst_path                = "${middleware_home_dir}/oracle_common/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
          default: {
            if ( $create_rcu == undef or $create_rcu ) {
              fail("Unkown domain_template(${domain_template}) for rcu with version ${version}")
            }
            $extensions_template_file = undef
            $wlst_path                = "${weblogic_home_dir}/common/bin"
            $template_em              = "${middleware_home_dir}/em/common/templates/wls/oracle.em_wls_template.jar"
            $template_jrf             = "${middleware_home_dir}/oracle_common/common/templates/wls/oracle.jrf_template.jar"
            $template_ohs             = "${middleware_home_dir}/ohs/common/templates/wls/ohs_managed_template.jar"
            $rcu_supported            = true
          }
        }
      }
      default: {
        fail("Version ${version} not recognized and supported.")
      }
    }

    $template_spaces     = "${middleware_home_dir}/Oracle_WC1/common/templates/applications/oracle.wc_spaces_template_11.1.1.jar"
    $template_bpm_spaces = "${middleware_home_dir}/Oracle_WC1/common/templates/applications/oracle.bpm.spaces_template_11.1.1.jar"
    $template_portlets   = "${middleware_home_dir}/Oracle_WC1/common/templates/applications/oracle.producer_apps_template_11.1.1.jar"
    $template_pagelet    = "${middleware_home_dir}/Oracle_WC1/common/templates/applications/oracle.pagelet-producer_template_11.1.1.jar"
    $template_discussion = "${middleware_home_dir}/Oracle_WC1/common/templates/applications/oracle.owc_discussions_template_11.1.1.jar"
    $template_ucm        = "${middleware_home_dir}/Oracle_WCC1/common/templates/applications/oracle.ucm.cs_template_11.1.1.jar"

    unless ( $domain_template == 'ohs_standalone' ) {
      $template_file = 'wls_install/domains/domain.py.erb'
    }

    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin"

    unless ( $log_dir ) {
      $admin_nodemanager_log_dir            = "${domain_dir}/servers/${adminserver_name}/logs"
      $node_mgr_log_dir                     = "${domain_dir}/nodemanager/nodemanager.log"
      $osb_nodemanager_log_dir              = "${domain_dir}/servers/osb_server1/logs"
      $soa_nodemanager_log_dir              = "${domain_dir}/servers/soa_server1/logs"
      $bam_nodemanager_log_dir              = "${domain_dir}/servers/bam_server1/logs"
      $ess_nodemanager_log_dir              = "${domain_dir}/servers/ess_server1/logs"
      $oim_nodemanager_log_dir              = "${domain_dir}/servers/oim_server1/logs"
      $oam_nodemanager_log_dir              = "${domain_dir}/servers/oam_server1/logs"
      $bi_nodemanager_log_dir               = "${domain_dir}/servers/bi_server1/logs"
      $wc_collaboration_nodemanager_log_dir = "${domain_dir}/servers/WC_Collaboration/logs"
      $wc_portlet_nodemanager_log_dir       = "${domain_dir}/servers/WC_Portlet/logs"
      $wc_spaces_nodemanager_log_dir        = "${domain_dir}/servers/WC_Spaces/logs"
      $umc_nodemanager_log_dir              = "${domain_dir}/servers/UCM_server1/logs"
    } else {
      $admin_nodemanager_log_dir = $osb_nodemanager_log_dir = $soa_nodemanager_log_dir = $bam_nodemanager_log_dir = $ess_nodemanager_log_dir = $oim_nodemanager_log_dir = $oam_nodemanager_log_dir = $bi_nodemanager_log_dir = $_c= $wc_portlet_nodemanager_log_dir = $wc_spaces_nodemanager_log_dir = $umc_nodemanager_log_dir = $log_dir
      $node_mgr_log_dir          = "${log_dir}/nodemanager_${domain_name}.log"

      # create all log folders
      unless ( defined(Exec["create ${log_dir} directory"]) ) {
        exec { "create ${log_dir} directory":
          command   => "mkdir -p ${log_dir}",
          unless    => "test -d ${log_dir}",
          user      => $puppet_os_user,
          path      => $exec_path,
          logoutput => $logoutput,
        }
      }

      unless ( defined(File[$log_dir]) ) {
        file { $log_dir:
          ensure  => directory,
          recurse => false,
          replace => false,
          require => Exec["create ${log_dir} directory"],
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }
      }
    }

    # the utils.py used by the wlst
    unless ( defined(File["${download_dir}/utils.py"]) ) {
      file { "${download_dir}/utils.py":
        ensure  => present,
        path    => "${download_dir}/utils.py",
        content => template('wls_install/domains/utils.py.erb'),
        replace => true,
        backup  => false,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
      }
    }

    if ( $extensions_template_file ) {
      file { "domain_extension.py ${domain_name} ${title}":
        ensure  => present,
        path    => "${download_dir}/domain_extension_${domain_name}.py",
        content => template($extensions_template_file),
        replace => true,
        backup  => false,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
        before  => File["domain.py ${domain_name} ${title}"],
      }
    }

    # the domain.py used by the wlst
    file { "domain.py ${domain_name} ${title}":
      ensure  => present,
      path    => "${download_dir}/domain_${domain_name}.py",
      content => template($template_file),
      replace => true,
      backup  => false,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
      require => File["${download_dir}/utils.py"],
    }

    if ( $domains_dir == "${middleware_home_dir}/user_projects/domains" ) {
      unless ( defined(File['weblogic_domain_folder']) ) {
        file { 'weblogic_domain_folder':
          ensure  => directory,
          path    => "${middleware_home_dir}/user_projects",
          recurse => false,
          replace => false,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }
        File['weblogic_domain_folder'] -> File[$domains_dir]
      }
    }

    unless ( defined(File[$domains_dir]) ) {
      file { $domains_dir:
        ensure  => directory,
        recurse => false,
        replace => false,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
      }
    }

    if ( $apps_dir ) {
      unless ( defined(File[$apps_dir]) ) {
        file { $apps_dir:
          ensure  => directory,
          recurse => false,
          replace => false,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }
      }
    }

    # FMW RCU only for wls 12.1.2 or higher and when template is not standard, adf_restricted or ohs_standalone
    if ( $rcu_supported ) {

      if  $create_rcu {

        # only works for a 12c middleware home
        # creates RCU for ADF
        if !( $rcu_database_url and $repository_sys_password and $repository_password and $repository_prefix ){
          fail('Not all RCU parameters are provided')
        }

        wls_install::utils::rcu{ "RCU_12c ${title}":
          version                     => $version,
          fmw_product                 => $rcu_domain_template,
          oracle_fmw_product_home_dir => "${middleware_home_dir}/oracle_common",
          jdk_home_dir                => $jdk_home_dir,
          os_user                     => $os_user,
          os_group                    => $os_group,
          download_dir                => $download_dir,
          rcu_action                  => 'create',
          rcu_jdbc_url                => $repository_database_url,
          rcu_database_url            => $rcu_database_url,
          rcu_sys_user                => $repository_sys_user,
          rcu_sys_password            => $repository_sys_password,
          rcu_prefix                  => $repository_prefix,
          rcu_password                => $repository_password,
          rcu_honor_omf               => $rcu_honor_omf,
          rcu_soa_profile             => $rcu_soa_profile,
          logoutput                   => $logoutput,
          before                      => Exec["execwlst ${domain_name} ${title}"],
        }
      }
    }

    # create domain
    # On newer WebLogic versions domain creation also create's a template nodemanager.properties
    # because we create the correct nodemanager.properties later in the process by wlsInstall::nodemanager
    # we remove it over here.
    #
    exec { "execwlst ${domain_name} ${title}":
      command     => "${wlst_path}/wlst.sh domain_${domain_name}.py; /bin/rm -f ${domain_dir}/nodemanager/nodemanager.properties",
      environment => "JAVA_HOME=${jdk_home_dir}",
      unless      => "/usr/bin/test -e ${domain_dir}",
      creates     => $domain_dir,
      cwd         => $download_dir,
      timeout     => 0,
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => [
        File["domain.py ${domain_name} ${title}"],
        File["${download_dir}/utils.py"],
        File[$domains_dir],
      ],
    }

    if ( $extensions_template_file ) {
      exec { "execwlst ${domain_name} extension ${title}":
        command     => "${wlst_path}/wlst.sh domain_extension_${domain_name}.py",
        environment => "JAVA_HOME=${jdk_home_dir}",
        cwd         => $download_dir,
        timeout     => 0,
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["execwlst ${domain_name} ${title}"],
          File["domain_extension.py ${domain_name} ${title}"],
        ],
      }
    }

    file { $wls_domains_file_location:
      ensure => 'present',
      mode   => '0600',
      owner  => 'root',
      group  => 'root',
    }

    file_line { "domain entry for ${domain_name}":
      path    => $wls_domains_file_location,
      line    => "${domain_name}:${domain_dir}",
      require => Exec["execwlst ${domain_name} ${title}"],
    }

    if ( $domain_template == 'oim' ) {

      file { "${download_dir}/${title}psa_opss_upgrade.rsp":
        ensure  => present,
        content => template('wls_install/oim/psa_opss_upgrade.rsp.erb'),
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
        backup  => false,
      }

      exec { "exec PSA OPSS store upgrade ${domain_name} ${title}":
        command   => "${middleware_home_dir}/oracle_common/bin/psa -response ${download_dir}/${title}psa_opss_upgrade.rsp",
        timeout   => 0,
        cwd       => $download_dir, # Added since psa binary saves and changes to current dir
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => [
          Exec["execwlst ${domain_name} ${title}"],
          Exec["execwlst ${domain_name} extension ${title}"],
          File["${download_dir}/${title}psa_opss_upgrade.rsp"],
        ],
      }

      exec { "execwlst create OPSS store ${domain_name} ${title}":
        command     => "${wlst_path}/wlst.sh ${middleware_home_dir}/Oracle_IDM1/common/tools/configureSecurityStore.py -d ${domain_dir} -m create -c IAM -p ${repository_password}",
        environment => "JAVA_HOME=${jdk_home_dir}",
        timeout     => 0,
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["execwlst ${domain_name} ${title}"],
          Exec["execwlst ${domain_name} extension ${title}"],
          Exec["exec PSA OPSS store upgrade ${domain_name} ${title}"],
        ],
      }

      exec { "execwlst validate OPSS store ${domain_name} ${title}":
        command     => "${wlst_path}/wlst.sh ${middleware_home_dir}/Oracle_IDM1/common/tools/configureSecurityStore.py -d ${domain_dir} -m validate",
        environment => "JAVA_HOME=${jdk_home_dir}",
        timeout     => 0,
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["execwlst ${domain_name} ${title}"],
          Exec["execwlst ${domain_name} extension ${title}"],
          Exec["execwlst create OPSS store ${domain_name} ${title}"],
        ],
      }

    }

    if $::kernel == 'SunOS' {

      if ( $domain_template =~ /^osb/ ){

        exec { "setDebugFlagOnFalse ${domain_name} ${title}":
          command   => "sed -e's/debugFlag=\"true\"/debugFlag=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh > /tmp/domain.tmp && mv /tmp/domain.tmp ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep debugFlag=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => [
            Exec["execwlst ${domain_name} ${title}"],
            Exec["execwlst ${domain_name} extension ${title}"],
          ],
        }

        exec { "setOSBDebugFlagOnFalse ${domain_name} ${title}":
          command   => "sed -e's/ALSB_DEBUG_FLAG=\"true\"/ALSB_DEBUG_FLAG=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh > /tmp/domain2.tmp && mv /tmp/domain2.tmp ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep ALSB_DEBUG_FLAG=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => Exec["setDebugFlagOnFalse ${domain_name} ${title}"],
        }

        exec { "setDERBY_FLAGOnFalse ${domain_name} ${title}":
          command   => "sed -e's/DERBY_FLAG=\"true\"/DERBY_FLAG=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh > /tmp/domain3.tmp && mv /tmp/domain3.tmp ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep DERBY_FLAG=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => Exec["setOSBDebugFlagOnFalse ${domain_name} ${title}"],
        }

      }

    } else {

      if ( $domain_template =~ /^osb/ ) {

        exec { "setDebugFlagOnFalse ${domain_name} ${title}":
          command   => "sed -i -e's/debugFlag=\"true\"/debugFlag=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep debugFlag=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => [
            Exec["execwlst ${domain_name} ${title}"],
            Exec["execwlst ${domain_name} extension ${title}"],
          ],
        }

        exec { "setOSBDebugFlagOnFalse ${domain_name} ${title}":
          command   => "sed -i -e's/ALSB_DEBUG_FLAG=\"true\"/ALSB_DEBUG_FLAG=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep ALSB_DEBUG_FLAG=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => Exec["setDebugFlagOnFalse ${domain_name} ${title}"],
        }

        exec { "setDERBY_FLAGOnFalse ${domain_name} ${title}":
          command   => "sed -i -e's/DERBY_FLAG=\"true\"/DERBY_FLAG=\"false\"/g' ${domain_dir}/bin/setDomainEnv.sh",
          onlyif    => "/bin/grep DERBY_FLAG=\"true\" ${domain_dir}/bin/setDomainEnv.sh | /usr/bin/wc -l",
          path      => $exec_path,
          user      => $os_user,
          group     => $os_group,
          logoutput => $logoutput,
          require   => Exec["setOSBDebugFlagOnFalse ${domain_name} ${title}"],
        }

      }
    }

    exec { "domain.py ${domain_name} ${title}":
      command   => "rm ${download_dir}/domain_${domain_name}.py",
      path      => $exec_path,
      user      => $os_user,
      group     => $os_group,
      logoutput => $logoutput,
      require   => Exec["execwlst ${domain_name} ${title}"],
    }

    if ( $extensions_template_file ) {
      exec { "domain.py ${domain_name} extension ${title}":
        command   => "rm ${download_dir}/domain_extension_${domain_name}.py",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => Exec["execwlst ${domain_name} extension ${title}"],
      }
    }
  }
}
