#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::increased_memory_bsu
#
# Patches the BSU utility to increase the amount of memory available for patches.
##
define wls_install::increased_memory_bsu (
  Stdlib::Absolutepath $middleware_home_dir = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath $jdk_home_dir        = lookup({name => 'wls_jdk_home_dir'}),
  String[1]            $os_user             = lookup({name => 'wls_os_user'}),
  Srting[1]            $os_group            = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath $download_dir        = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput           = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                    $log_output          = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  $exec_path = "/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:${jdk_home_dir}/bin"

  exec { "change memory params for ${name}":
    command   => "sed -e's/MEM_ARGS=\"-Xms256m -Xmx512m\"/MEM_ARGS=\"-Xms256m -Xmx1024m -XX:-UseGCOverheadLimit\"/g' ${middleware_home_dir}/utils/bsu/bsu.sh > ${download_dir}/bsu.sh && mv ${download_dir}/bsu.sh ${middleware_home_dir}/utils/bsu/bsu.sh;chmod +x ${middleware_home_dir}/utils/bsu/bsu.sh",
    unless    => "grep 'MEM_ARGS=\"-Xms256m -Xmx1024m -XX:-UseGCOverheadLimit\"' ${middleware_home_dir}/utils/bsu/bsu.sh",
    path      => $exec_path,
    user      => $os_user,
    group     => $os_group,
    cwd       => $download_dir,
    logoutput => $logoutput,
  }
}
