#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::nodemanager
#
# install and configures the nodemanager
#
define wls_install::nodemanager (
  Wls_install::Versions $version                               = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath  $middleware_home_dir                   = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $weblogic_home_dir                     = lookup({name => 'wls_weblogic_home_dir'}),
  Integer               $nodemanager_port                      = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  String[1]             $nodemanager_address                   = undef,
  Boolean               $nodemanager_secure_listener           = true,
  Boolean               $jsse_enabled                          = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean               $custom_trust                          = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]   $trust_keystore_file                   = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[String[1]]   $trust_keystore_passphrase             = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  Boolean               $custom_identity                       = false,
  Optional[String[1]]   $custom_identity_keystore_filename     = undef,
  Optional[String[1]]   $custom_identity_keystore_passphrase   = undef,
  Optional[String[1]]   $custom_identity_alias                 = undef,
  Optional[String[1]]   $custom_identity_privatekey_passphrase = undef,
  String                $extra_arguments                       = '',
  Optional[
    Stdlib::Absolutepath
  ]                     $wls_domains_dir                       = lookup({name => 'wls_domains_dir', default_value => undef}),
  Optional[
    String[1]]          $domain_name                           = lookup({name => 'domain_name', default_value => undef}),
  Stdlib::Absolutepath  $jdk_home_dir                          = lookup({name => 'wls_jdk_home_dir'}),
  String[1]             $os_user                               = lookup({name => 'wls_os_user'}),
  String[1]             $os_group                              = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir                          = lookup({name => 'wls_download_dir'}),
  Optional[
    Stdlib::Absolutepath
  ]                     $log_dir                               = lookup({name => 'wls_log_dir', default_value => undef}),
  String[1]             $log_file                              = 'nodemanager.log',
  Integer               $sleep                                 = lookup({name => 'wls_nodemanager_sleep', default_value => 20}),
  Hash                  $properties                            = {},
  Boolean               $ohs_standalone                        = false,
  String[1]             $puppet_os_user                        = lookup({name => 'puppet_os_user', default_value => 'root'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput                             = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                     $log_output                            = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  case $version {
    1036, 1111, 1211: {
      $node_mgr_home = "${weblogic_home_dir}/common/nodemanager"
      $start_home = "${weblogic_home_dir}/server/bin"
      $nodemanager_property_version = '10.3'
    }
    1212: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home = "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.1.2'
    }
    1213: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home = "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.1.3'
    }
    1221: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home = "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.2.1'
    }
    12211: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home= "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.2.1.1.0'
    }
    12212: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home= "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.2.1.2.0'
    }
    12213: {
      $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
      $start_home= "${domains_dir}/${domain_name}/bin"
      $nodemanager_property_version = '12.2.1.3.0'
    }
    default: {
      $node_mgr_home = "${weblogic_home_dir}/common/nodemanager"
      $start_home   = "${weblogic_home_dir}/server/bin"
    }
  }

  $exec_path    = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  unless ( $log_dir ) {
    $node_mgr_log_dir = "${node_mgr_home}/${log_file}"
  } else {
      # create all folders
      unless ( defined(Exec["create ${log_dir} directory"]) ) {
        exec { "create ${log_dir} directory":
          command => "mkdir -p ${log_dir}",
          unless  => "test -d ${log_dir}",
          user    => $puppet_os_user,
          path    => $exec_path,
          group   => $os_group,
          cwd     => $node_mgr_home,
        }
      }
      unless ( defined(File[$log_dir]) ) {
        file { $log_dir:
          ensure  => directory,
          recurse => false,
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          require => Exec["create ${log_dir} directory"],
          before  => Exec["startNodemanager ${title}"],
        }
      }
      $node_mgr_log_dir = "${log_dir}/${log_file}"
  }

  case $::kernel {
    'Linux': {
      if ( $version >= 1212 ){
        $check_command = "ps -wwo pid,cmd -p $(netstat -tnlp 2>/dev/null | /bin/grep \":${nodemanager_port} \" | /usr/bin/awk '{print \$7}' | /usr/bin/awk -F '/' {'print \$1}' |uniq) 2>/dev/null | /bin/grep 'weblogic.NodeManager' | /bin/grep ${domain_name}"
      } else {
        $check_command = "ps -wwo pid,cmd -p $(netstat -tnlp 2>/dev/null | /bin/grep \":${nodemanager_port} \" | /usr/bin/awk '{print \$7}' | /usr/bin/awk -F '/' {'print \$1}' |uniq) 2>/dev/null | /bin/grep 'weblogic.NodeManager'"
      }
      $native_lib        = 'linux/x86_64'
      $su_command        = "su -l ${os_user}"
      $java_statement    = 'java'
      $netstat_statement = "/bin/netstat -lnt | /bin/grep ':${nodemanager_port}'"
    }
    'SunOS': {
      case $::kernelrelease {
        '5.11': {
          if ( $version >= 1212 ){
            $check_command = "/bin/ps wwxa | /bin/grep -v grep | /bin/grep 'weblogic.NodeManager' | /bin/grep ${domain_name}"
          } else {
            $check_command = '/bin/ps wwxa | /bin/grep -v grep | /bin/grep \'weblogic.NodeManager\''
          }
        }
        default: {
          if ( $version >= 1212 ){
            $check_command = "/usr/ucb/ps wwxa | /bin/grep -v grep | /bin/grep 'weblogic.NodeManager' | /bin/grep ${domain_name}"
          } else {
            $check_command = '/usr/ucb/ps wwxa | /bin/grep -v grep | /bin/grep \'weblogic.NodeManager\''
          }
        }
      }
      $native_lib        = 'solaris/x64'
      $su_command        = "su - ${os_user}"
      $java_statement    = 'java -d64'
      $netstat_statement = "/bin/netstat -an -P tcp | /bin/grep LISTEN | /bin/grep '.${nodemanager_port}'"
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux or Solaris host")
    }
  }

  Exec {
    logoutput => $logoutput,
  }

  # do it once but don't replace it because of encrypted trust passwords
  if ( $custom_identity ) {
    $replace_nodemanager_properties = false
  } else {
    $replace_nodemanager_properties = true
  }

  $property_defaults = {
    'properties_version'                 => $nodemanager_property_version,
    'log_limit'                          => 0,
    'domains_dir_remote_sharing_enabled' => false,
    'authentication_enabled'             => true,
    'log_level'                          => 'INFO',
    'domains_file_enabled'               => true,
    'start_script_name'                  => 'startWebLogic.sh',
    'native_version_enabled'             => true,
    'log_to_stderr'                      => true,
    'log_count'                          => '1',
    'domain_registration_enabled'        => false,
    'stop_script_enabled'                => true,
    'quit_enabled'                       => false,
    'log_append'                         => true,
    'state_check_interval'               => 500,
    'crash_recovery_enabled'             => true,
    'start_script_enabled'               => true,
    'log_formatter'                      => 'weblogic.nodemanager.server.LogFormatter',
    'listen_backlog'                     => 50,
  }

  $properties_merged = merge($property_defaults, $properties)

  # nodemanager is part of the domain creation
  $properties_file_title = "nodemanager.properties ux ${version} ${title}"
  if ( $version <= 1211 ) {
    file { $properties_file_title:
      ensure  => present,
      path    => "${node_mgr_home}/nodemanager.properties",
      replace => $replace_nodemanager_properties,
      content => template('wls_install/nodemgr/nodemanager.properties.erb'),
      owner   => $os_user,
      group   => $os_group,
      mode    => '0600',
      before  => Exec["startNodemanager ${title}"],
    }
  } else {
    if ( $version >= 1221 ) {
      $new_version = 1221
    } else {
      $new_version = $version
    }

    # do not replace when it contains encrypted custom trust field like CustomIdentityKeyStorePassPhrase
    file { $properties_file_title:
      ensure  => present,
      path    => "${node_mgr_home}/nodemanager.properties",
      replace => $replace_nodemanager_properties,
      content => template("wls_install/nodemgr/nodemanager.properties_${new_version}.erb"),
      owner   => $os_user,
      group   => $os_group,
      mode    => '0600',
      before  => Exec["startNodemanager ${title}"],
    }
  }

  if ( $custom_trust ) {
    $trust_env = "-Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=${trust_keystore_file} -Dweblogic.security.CustomTrustKeystorePassPhrase=${trust_keystore_passphrase}"
  } else {
    $trust_env = ''
  }

  if ( $jsse_enabled ) {
    $env = "JAVA_OPTIONS=-Dweblogic.ssl.JSSEEnabled=true -Dweblogic.security.SSL.enableJSSE=true ${trust_env} ${extra_arguments}"
  } else {
    $env = "JAVA_OPTIONS=-Dweblogic.ssl.JSSEEnabled=false -Dweblogic.security.SSL.enableJSSE=false ${trust_env} ${extra_arguments}"
  }

  $start_command = "su - ${os_user} -c \"nohup ${start_home}/startNodeManager.sh >${log_dir}/nodemanager.out 2>&1 &\""
  $restart_command = "kill $(${check_command} | awk '{print \$1}'); sleep 1; ${start_command}"

  exec { "startNodemanager ${title}":
    command     => $start_command,
    environment => [
      $env,
      "JAVA_HOME=${jdk_home_dir}",
      'JAVA_VENDOR=Oracle'
    ],
    unless      => $check_command,
    path        => $exec_path,
    cwd         => $node_mgr_home,
    logoutput   => $logoutput,
  }

  exec {"restart NodeManager ${title}":
    command     => $restart_command,
    environment => [
      $env,
      "JAVA_HOME=${jdk_home_dir}",
      'JAVA_VENDOR=Oracle'
    ],
    onlyif      => $check_command,
    path        => $exec_path,
    cwd         => $node_mgr_home,
    refreshonly => true,
    subscribe   => File[$properties_file_title],
    logoutput   => $logoutput,
  }

  # using fiddyspence/sleep module
  sleep { "wake up ${title}":
    bedtime       => $sleep,
    wakeupfor     => $netstat_statement,
    dozetime      => 2,
    failontimeout => true,
    subscribe     => [
      Exec["startNodemanager ${title}"],
      Exec["restart NodeManager ${title}"]
    ],
    refreshonly   => true,
  }
}
