#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::opatch
#
# installs oracle patches for Oracle products
#
##
define wls_install::opatch(
  Enum[
    present,
    absent]            $ensure                  = present,
  Stdlib::Absolutepath $oracle_product_home_dir = undef,
  Stdlib::Absolutepath $jdk_home_dir            = lookup({name => 'wls_jdk_home_dir', default_value => undef}),
  Integer              $patch_id                = undef,
  String[1]            $patch_file              = undef,
  String[1]            $os_user                 = lookup({name => 'wls_os_user', default_value => undef}),
  String[1]            $os_group                = lookup({name => 'wls_os_group', default_value => undef}),
  Stdlib::Absolutepath $download_dir            = lookup({name => 'wls_download_dir', default_value => undef}),
  String[1]            $source                  = lookup({name => 'wls_source', default_value => undef}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput               = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                    $log_output              = undef, # DEPRECATED
  Optional[Boolean]    $remote_file             = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  if ( $source ) {
    $mountPoint = $source
  } else {
    $mountPoint = 'puppet:///modules/wls_install/'
  }

  if ( $ensure == present ) {
    archive { "${download_dir}/${patch_file}":
      ensure  => present,
      cleanup => false,
      extract => false,
      source  => "${mountPoint}/${patch_file}",
      user    => $os_user,
      group   => $os_group,
      before  => Wls_opatch["${oracle_product_home_dir}:${patch_id}"],
    }
    $disk1_file = "${download_dir}/${patch_file}"
  } else {
    $disk1_file = undef
  }

  case $::kernel {
    'Linux': {
      $oraInstPath = '/etc'
    }
    'SunOS': {
      $oraInstPath = '/var/opt/oracle'
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }
  }

  wls_opatch{ "${oracle_product_home_dir}:${patch_id}":
    ensure       => $ensure,
    os_user      => $os_user,
    source       => $disk1_file,
    jdk_home_dir => $jdk_home_dir,
    orainst_dir  => $oraInstPath,
    tmp_dir      => $download_dir,
  }
}
