#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::packdomain
#
# pack a new weblogic domain
##
define wls_install::packdomain (
  Wls_install::Versions $version             = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath  $weblogic_home_dir   = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $jdk_home_dir        = lookup({name => 'wls_jdk_home_dir'}),
  Stdlib::Absolutepath  $wls_domains_dir     = lookup({name => 'wls_domains_dir', default_value => undef}),
  String[1]             $domain_name         = lookup({name => 'domain_name'}),
  String[1]             $os_user             = lookup({name => 'wls_os_user'}),
  String[1]             $os_group            = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir        = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput           = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                     $log_output          = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  if ( $version >= 1221  ) {
    $bin_dir = "${middleware_home_dir}/oracle_common/common/bin/pack.sh"
  } else {
    $bin_dir = "${weblogic_home_dir}/common/bin/pack.sh"
  }

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin"
  $pack_command = "${bin_dir} -domain=${domains_dir}/${domain_name} -template=${download_dir}/domain_${domain_name}.jar -template_name=domain_${domain_name} -log=${download_dir}/domain_${domain_name}.log -log_priority=INFO -managed=true"

  exec { "pack domain ${domain_name} ${title}":
    command   => $pack_command,
    creates   => "${download_dir}/domain_${domain_name}.jar",
    path      => $exec_path,
    user      => $os_user,
    group     => $os_group,
    logoutput => $logoutput,
  }
}
