#
# See the file "LICENSE" for the full license governing this code.
#
##
# This class allows you to install the WebLogic software on your system. It allows
# one version te be installed (hence it's a class). If you want to install more then
# one version, you'll need to use the defined type wls_install::software_version.
#
class wls_install::software (
  Wls_install::Versions          $version              = 1111,
  String[1]                      $filename             = undef,
  Stdlib::Absolutepath           $oracle_base_home_dir = undef,
  Stdlib::Absolutepath           $middleware_home_dir  = undef,
  Optional[Stdlib::Absolutepath] $weblogic_home_dir    = undef,
  Optional[Stdlib::Absolutepath] $wls_domains_dir      = undef,
  Optional[Stdlib::Absolutepath] $wls_apps_dir         = undef,
  Boolean                        $fmw_infra            = false,
  Stdlib::Absolutepath           $jdk_home_dir         = undef,
  String[1]                      $os_user              = undef,
  String[1]                      $os_group             = undef,
  Stdlib::Absolutepath           $download_dir         = undef,
  String[1]                      $source               = undef,
  Optional[String]               $java_parameters      = '',
  Stdlib::Absolutepath           $temp_directory       = '/tmp',
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                              $logoutput            = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                              $log_output           = undef, # DEPRECATED
  Optional[Boolean]              $remote_file          = undef, # DEPRECATED
) {
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  wls_install::software_version{ 'base':
    version              => $version,
    filename             => $filename,
    oracle_base_home_dir => $oracle_base_home_dir,
    middleware_home_dir  => $middleware_home_dir,
    weblogic_home_dir    => $weblogic_home_dir,
    wls_domains_dir      => $wls_domains_dir,
    wls_apps_dir         => $wls_apps_dir,
    fmw_infra            => $fmw_infra,
    jdk_home_dir         => $jdk_home_dir,
    os_user              => $os_user,
    os_group             => $os_group,
    download_dir         => $download_dir,
    source               => $source,
    java_parameters      => $java_parameters,
    logoutput            => $logoutput,
    temp_directory       => $temp_directory,
  }
}
