#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::storeuserconfig
#
# generic storeuserconfig wlst script
#
define wls_install::storeuserconfig (
  String[1]            $domain_name         = lookup({name => 'domain_name'}),
  Stdlib::Absolutepath $weblogic_home_dir   = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath $jdk_home_dir        = lookup({name => 'wls_jdk_home_dir'}),
  String[1]            $adminserver_address = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer              $adminserver_port    = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Stdlib::Absolutepath $user_config_dir     = undef,
  String[1]            $weblogic_user       = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]            $weblogic_password   = lookup({name => 'domain_wls_password'}),
  String[1]            $os_user             = lookup({name => 'wls_os_user'}),
  String[1]            $os_group            = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath $download_dir        = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput           = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                    $log_output          = undef, # DEPRECATED
  Optional[Boolean]    $remote_file         = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  # the py script used by the wlst*-
  file { "${download_dir}/${title}storeUserConfig.py":
    ensure  => present,
    path    => "${download_dir}/${title}storeUserConfig.py",
    content => template('wls_install/wlst/storeUserConfig.py.erb'),
    backup  => false,
    replace => true,
    mode    => '0555',
    owner   => $os_user,
    group   => $os_group,
  }

  $javaCommand = 'java -Dweblogic.management.confirmKeyfileCreation=true -Dweblogic.security.SSL.ignoreHostnameVerification=true weblogic.WLST -skipWLSModuleScanning '
  $exec_path   = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  exec { "execwlst ${title}storeUserConfig.py":
    command     => "${javaCommand} ${download_dir}/${title}storeUserConfig.py ${weblogic_password}",
    environment => [
      "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
      "JAVA_HOME=${jdk_home_dir}"
    ],
    unless      => "ls -l ${user_config_dir}/${os_user}-${domain_name}-WebLogicConfig.properties",
    path        => $exec_path,
    user        => $os_user,
    group       => $os_group,
    logoutput   => $logoutput,
    require     => File["${download_dir}/${title}storeUserConfig.py"],
  }

}
