#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::utils::fmwcluster
#
# transform domain to a soa,osb,bam cluster
##
define wls_install::utils::fmwcluster (
  Wls_install::Versions $version                     = lookup({name => 'wls_version', default_value => 1111}),
  Variant[
    Integer[1116,1116],
    Integer[1117,1117]
  ]                     $ofm_version                 = lookup({name => 'ofm_version', default_value => 1117}),
  Stdlib::Absolutepath  $weblogic_home_dir           = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir         = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $jdk_home_dir                = lookup({name => 'wls_jdk_home_dir'}),
  Stdlib::Absolutepath  $wls_domains_dir             = lookup({name => 'wls_domains_dir', default_value => undef}),
  String[1]             $domain_name                 = lookup({name => 'domain_name'}),
  String[1]             $adminserver_name            = lookup({name => 'domain_adminserver', default_value => 'AdminServer'}),
  String[1]             $adminserver_address         = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer               $adminserver_port            = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Integer               $nodemanager_port            = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  Optional[String[1]]   $soa_cluster_name            = undef,
  Optional[String[1]]   $bam_cluster_name            = undef,
  Optional[String[1]]   $osb_cluster_name            = undef,
  Optional[String[1]]   $oam_cluster_name            = undef,
  Optional[String[1]]   $oim_cluster_name            = undef,
  Optional[String[1]]   $ess_cluster_name            = undef,
  Optional[String[1]]   $bi_cluster_name             = undef,
  Boolean               $bpm_enabled                 = false,
  Boolean               $bam_enabled                 = false,
  Boolean               $osb_enabled                 = false,
  Boolean               $soa_enabled                 = false,
  Boolean               $oam_enabled                 = false,
  Boolean               $oim_enabled                 = false,
  Boolean               $b2b_enabled                 = false,
  Boolean               $ess_enabled                 = false,
  Boolean               $bi_enabled                  = false,
  String[1]             $repository_prefix           = lookup({name => 'repository_prefix', default_value => 'DEV'}),
  String[1]             $weblogic_user               = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]             $weblogic_password           = lookup({name => 'domain_wls_password'}),
  String[1]             $os_user                     = lookup({name => 'wls_os_user'}),
  String[1]             $os_group                    = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir                = lookup({name => 'wls_download_dir'}),
  Boolean               $retain_file_store           = lookup({name => 'retain_security_file_store', default_value => false}),
  Boolean               $jsse_enabled                = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean               $custom_trust                = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]   $trust_keystore_file         = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[String[1]]   $trust_keystore_passphrase   = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  Boolean               $nodemanager_secure_listener = true,
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum['on_failure']
    ]
  ]                     $log_output                  = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir == undef or $wls_domains_dir == '') {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  } else {
    $domains_dir = $wls_domains_dir
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  if ( $soa_enabled ) {
    # check if the soa is already targeted to the cluster on this weblogic domain
    $soa_found = wls_install::soa_cluster_configured($domain_dir, $soa_cluster_name)

    if ( $soa_found ) {
      $convert_soa = true
    } else {
      $convert_soa = false
    }
  } else {
    $convert_soa = false
  }

  if ( $osb_enabled ) {
    # check if the soa is already targeted to the cluster on this weblogic domain
    if ( wls_install::osb_cluster_configured($domain_dir, $osb_cluster_name) ) {
      $convert_osb = true
    } else {
      $convert_osb = false
    }
  } else {
    $convert_osb = false
  }

  if ( $bam_enabled ) {
    # check if the soa is already targeted to the cluster on this weblogic domain
    if ( wls_install::bam_cluster_configured($domain_dir, $bam_cluster_name) ) {
      $convert_bam = true
    } else {
      $convert_bam = false
    }
  } else {
    $convert_bam = false
  }

  if ( $convert_soa or $convert_osb or $convert_bam ) {

    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

    if ( $version == 1213 or $version >= 1221 ) {
      #shutdown adminserver for offline WLST scripts
      wls_install::control{"ShutdownAdminServerForSoa${title}":
        middleware_home_dir         => $middleware_home_dir,
        weblogic_home_dir           => $weblogic_home_dir,
        jdk_home_dir                => $jdk_home_dir,
        wls_domains_dir             => $domains_dir,
        domain_name                 => $domain_name,
        server_type                 => 'admin',
        target                      => 'Server',
        server                      => $adminserver_name,
        adminserver_address         => $adminserver_address,
        adminserver_port            => $adminserver_port,
        nodemanager_secure_listener => $nodemanager_secure_listener,
        nodemanager_port            => $nodemanager_port,
        jsse_enabled                => $jsse_enabled,
        custom_trust                => $custom_trust,
        trust_keystore_file         => $trust_keystore_file,
        trust_keystore_passphrase   => $trust_keystore_passphrase,
        action                      => 'stop',
        weblogic_user               => $weblogic_user,
        weblogic_password           => $weblogic_password,
        os_user                     => $os_user,
        os_group                    => $os_group,
        download_dir                => $download_dir,
        logoutput                   => $logoutput,
      }
      if $version >= 1221 {
        $new_version = 1221
      } else {
        $new_version = $version
      }

      file { "${download_dir}/assignOsbSoaBpmBamToClusters${title}.py":
        ensure  => present,
        content => template("wls_install/wlst/wlstexec/fmw/assignOsbSoaBpmBamToClusters_${new_version}.py.erb"),
        backup  => false,
        replace => true,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
      }

      # reorder all apps,libraries, startup , shutdown and datasources
      exec { "execwlst assignOsbSoaBpmBamToClusters.py ${title}":
        command   => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/assignOsbSoaBpmBamToClusters${title}.py",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        timeout   => 0,
        logoutput => $logoutput,
        require   => [
          File["${download_dir}/assignOsbSoaBpmBamToClusters${title}.py"],
          Wls_install::Control["ShutdownAdminServerForSoa${title}"]
        ],
      }
      -> wls_install::utils::private::old_servers_removed { $domain_dir:
        bam_enabled => $bam_enabled,
        osb_enabled => $osb_enabled,
        soa_enabled => $soa_enabled,
        ess_enabled => $ess_enabled,
      }

      #startup adminserver for offline WLST scripts
      wls_install::control{"StartupAdminServerForSoa${title}":
        middleware_home_dir         => $middleware_home_dir,
        weblogic_home_dir           => $weblogic_home_dir,
        jdk_home_dir                => $jdk_home_dir,
        wls_domains_dir             => $domains_dir,
        domain_name                 => $domain_name,
        server_type                 => 'admin',
        target                      => 'Server',
        server                      => $adminserver_name,
        adminserver_address         => $adminserver_address,
        adminserver_port            => $adminserver_port,
        nodemanager_secure_listener => $nodemanager_secure_listener,
        nodemanager_port            => $nodemanager_port,
        action                      => 'start',
        jsse_enabled                => $jsse_enabled,
        custom_trust                => $custom_trust,
        trust_keystore_file         => $trust_keystore_file,
        trust_keystore_passphrase   => $trust_keystore_passphrase,
        weblogic_user               => $weblogic_user,
        weblogic_password           => $weblogic_password,
        os_user                     => $os_user,
        os_group                    => $os_group,
        download_dir                => $download_dir,
        logoutput                   => $logoutput,
        require                     => Exec["execwlst assignOsbSoaBpmBamToClusters.py ${title}"],
      }
    }
    elsif ( $version <= 1111 ) {

      case $::kernel {
        'Linux': {
          $java_statement = 'java'
        }
        'SunOS': {
          $java_statement = 'java -d64'
        }
        default: {
          fail("Unrecognized operating system ${::kernel}")
        }
      }
      $javaCommand = "${java_statement} -Dweblogic.security.SSL.ignoreHostnameVerification=true weblogic.WLST -skipWLSModuleScanning "

      if ( $osb_enabled ) {
        $last_step = "execwlst osb-createUDD.py ${title}"
      } elsif ( $oim_enabled ) {
        $last_step = "execwlst oim-createUDD.py ${title}"
      } else {
        $last_step = "execwlst soa-bpm-createUDD.py ${title}"
      }

      # only for soa suite 1036 or 1111 and not if OAM is already installed
      if ( $soa_enabled and !$oam_enabled and !$retain_file_store ) {
        # the domain.py used by the wlst
        file { "migrateSecurityStore.py ${domain_name} ${title}":
          ensure  => present,
          path    => "${download_dir}/migrateSecurityStore_${domain_name}.py",
          content => template('wls_install/wlst/wlstexec/fmw/migrateSecurityStore.py.erb'),
          replace => true,
          backup  => false,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }
        exec { "execwlst create OPSS store ${domain_name} ${title}":
          command     => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/migrateSecurityStore_${domain_name}.py ${weblogic_password}",
          environment => "JAVA_HOME=${jdk_home_dir}",
          require     => File["migrateSecurityStore.py ${domain_name} ${title}"],
          timeout     => 0,
          before      => Wls_install::Control["ShutdownAdminServerForSoa${title}"],
          logoutput   => $logoutput,
        }
      }

      #shutdown adminserver for offline WLST scripts
      wls_install::control{"ShutdownAdminServerForSoa${title}":
        middleware_home_dir         => $middleware_home_dir,
        weblogic_home_dir           => $weblogic_home_dir,
        jdk_home_dir                => $jdk_home_dir,
        wls_domains_dir             => $domains_dir,
        domain_name                 => $domain_name,
        server_type                 => 'admin',
        target                      => 'Server',
        server                      => $adminserver_name,
        adminserver_address         => $adminserver_address,
        adminserver_port            => $adminserver_port,
        nodemanager_secure_listener => $nodemanager_secure_listener,
        nodemanager_port            => $nodemanager_port,
        action                      => 'stop',
        jsse_enabled                => $jsse_enabled,
        custom_trust                => $custom_trust,
        trust_keystore_file         => $trust_keystore_file,
        trust_keystore_passphrase   => $trust_keystore_passphrase,
        weblogic_user               => $weblogic_user,
        weblogic_password           => $weblogic_password,
        os_user                     => $os_user,
        os_group                    => $os_group,
        download_dir                => $download_dir,
        logoutput                   => $logoutput,
      }

      file { "${download_dir}/assignOsbSoaBpmBamToClusters${title}.py":
        ensure  => present,
        content => template('wls_install/wlst/wlstexec/fmw/assignOsbSoaBpmBamToClusters.py.erb'),
        backup  => false,
        replace => true,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
      }

      # reorder all apps,libraries, startup , shutdown and datasources
      exec { "execwlst assignOsbSoaBpmBamToClusters.py ${title}":
        command     => "${javaCommand} ${download_dir}/assignOsbSoaBpmBamToClusters${title}.py",
        environment => [
          "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
          "JAVA_HOME=${jdk_home_dir}"
        ],
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          File["${download_dir}/assignOsbSoaBpmBamToClusters${title}.py"],
          Wls_install::Control["ShutdownAdminServerForSoa${title}"]
        ],
      }
      -> wls_install::utils::private::old_servers_removed { $domain_dir:
        bam_enabled => $bam_enabled,
        osb_enabled => $osb_enabled,
        soa_enabled => $soa_enabled,
        ess_enabled => $ess_enabled,
      }

      if ( $soa_enabled ) {

        if ( $bam_enabled ) {
          $action = "--bamcluster ${bam_cluster_name} --soacluster ${soa_cluster_name}"
        } else {
          $action = "--soacluster ${soa_cluster_name}"
        }

        file { "${download_dir}/soa-createUDD${title}.py":
          ensure  => present,
          content => template('wls_install/wlst/wlstexec/fmw/soa-createUDD.py.erb'),
          backup  => false,
          replace => true,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }

        # execute WLST script
        exec { "execwlst soa-createUDD.py ${title}":
          command     => "${javaCommand} ${download_dir}/soa-createUDD${title}.py --domain_home ${domain_dir} ${action} --create_jms true",
          environment => [
            "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
            "JAVA_HOME=${jdk_home_dir}"
          ],
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [
            Wls_install::Control["ShutdownAdminServerForSoa${title}"],
            File["${download_dir}/soa-createUDD${title}.py"],
            Exec["execwlst assignOsbSoaBpmBamToClusters.py ${title}"]
          ],
        }
        # the py script used by the wlst
        file { "${download_dir}/soa-bpm-createUDD${title}.py":
          ensure  => present,
          content => template('wls_install/wlst/wlstexec/fmw/soa-bpm-createUDD.py.erb'),
          backup  => false,
          replace => true,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }

        # execute WLST script
        exec { "execwlst soa-bpm-createUDD.py ${title}":
          command     => "${javaCommand} ${download_dir}/soa-bpm-createUDD${title}.py",
          environment => [
            "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
            "JAVA_HOME=${jdk_home_dir}"
          ],
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [
            File["${download_dir}/soa-bpm-createUDD${title}.py"],
            Wls_install::Control["ShutdownAdminServerForSoa${title}"],
            Exec["execwlst assignOsbSoaBpmBamToClusters.py ${title}"],
            Exec["execwlst soa-createUDD.py ${title}"]
          ],
        }

        if( $oim_enabled == true ) {
          # the py script used by the wlst
          file { "${download_dir}/oim-createUDD${title}.py":
            ensure  => present,
            content => template('wls_install/wlst/wlstexec/fmw/oim-createUDD.py.erb'),
            backup  => false,
            replace => true,
            mode    => '0775',
            owner   => $os_user,
            group   => $os_group,
          }

          # execute WLST script
          exec { "execwlst oim-createUDD.py ${title}":
            command     => "${javaCommand} ${download_dir}/oim-createUDD${title}.py",
            environment => [
              "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
              "JAVA_HOME=${jdk_home_dir}"
            ],
            path        => $exec_path,
            user        => $os_user,
            group       => $os_group,
            logoutput   => $logoutput,
            require     => [
              File["${download_dir}/oim-createUDD${title}.py"],
              Exec["execwlst soa-bpm-createUDD.py ${title}"]
            ],
          }
        }
      }

      if ( $osb_enabled ) {

        if ( $soa_enabled ) {
          $last_soa_step = "execwlst soa-bpm-createUDD.py ${title}"
        } else  {
          $last_soa_step = "execwlst assignOsbSoaBpmBamToClusters.py ${title}"
        }

        # the py script used by the wlst
        file { "${download_dir}/osb-createUDD${title}.py":
          ensure  => present,
          content => template('wls_install/wlst/wlstexec/fmw/osb-createUDD.py.erb'),
          backup  => false,
          replace => true,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }

        # execute WLST script
        exec { "execwlst osb-createUDD.py ${title}":
          command     => "${javaCommand} ${download_dir}/osb-createUDD${title}.py",
          environment => [
            "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
            "JAVA_HOME=${jdk_home_dir}"
          ],
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [
            File["${download_dir}/osb-createUDD${title}.py"],
            Wls_install::Control["ShutdownAdminServerForSoa${title}"],
            Exec[$last_soa_step]
          ],
        }
      }

      #startup adminserver for offline WLST scripts
      wls_install::control{ "StartupAdminServerForSoa${title}":
        middleware_home_dir         => $middleware_home_dir,
        weblogic_home_dir           => $weblogic_home_dir,
        jdk_home_dir                => $jdk_home_dir,
        wls_domains_dir             => $domains_dir,
        domain_name                 => $domain_name,
        server_type                 => 'admin',
        target                      => 'Server',
        server                      => $adminserver_name,
        adminserver_address         => $adminserver_address,
        adminserver_port            => $adminserver_port,
        nodemanager_secure_listener => $nodemanager_secure_listener,
        nodemanager_port            => $nodemanager_port,
        action                      => 'start',
        jsse_enabled                => $jsse_enabled,
        custom_trust                => $custom_trust,
        trust_keystore_file         => $trust_keystore_file,
        trust_keystore_passphrase   => $trust_keystore_passphrase,
        weblogic_user               => $weblogic_user,
        weblogic_password           => $weblogic_password,
        os_user                     => $os_user,
        os_group                    => $os_group,
        download_dir                => $download_dir,
        logoutput                   => $logoutput,
        require                     => Exec[$last_step],
      }

      # the py script used by the wlst
      file { "${download_dir}/changeWorkmanagers${title}.py":
        ensure  => present,
        content => template('wls_install/wlst/wlstexec/fmw/changeWorkmanagers.py.erb'),
        backup  => false,
        replace => true,
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
      }

      # execute WLST script
      exec { "execwlst changeWorkmanagers.py ${title}":
        command     => "${javaCommand} ${download_dir}/changeWorkmanagers${title}.py ${weblogic_password}",
        environment => [
          "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
          "JAVA_HOME=${jdk_home_dir}"
        ],
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          File["${download_dir}/changeWorkmanagers${title}.py"],
          Wls_install::Control["StartupAdminServerForSoa${title}"]
        ],
      }

      if ( $oim_enabled ) {
        # the py script used by the wlst
        file { "${download_dir}/changeWorkmanagersOim${title}.py":
          ensure  => present,
          content => template('wls_install/wlst/wlstexec/fmw/changeWorkmanagersOim.py.erb'),
          backup  => false,
          replace => true,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }

        # execute WLST script
        exec { "execwlst changeWorkmanagersOim.py ${title}":
          command     => "${javaCommand} ${download_dir}/changeWorkmanagersOim${title}.py ${weblogic_password}",
          environment => [
            "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
            "JAVA_HOME=${jdk_home_dir}"
          ],
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [
            File["${download_dir}/changeWorkmanagersOim${title}.py"],
            Wls_install::Control["StartupAdminServerForSoa${title}"],
            Exec["execwlst changeWorkmanagers.py ${title}"]
          ],
        }
      }
    }
    else {
      fail('unknown version')
    }
  }
}
