#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::utils::fmwclusterjrf
#
# transform domain to a ADF cluster
##
define wls_install::utils::fmwclusterjrf (
  Wls_install::Versions $version                    = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath  $weblogic_home_dir          = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir        = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $jdk_home_dir               = lookup({name => 'wls_jdk_home_dir'}),
  String[1]             $domain_name                = lookup({name => 'domain_name'}),
  Stdlib::Absolutepath  $wls_domains_dir            = lookup({name => 'wls_domains_dir', default_value => undef}),
  String[1]             $adminserver_name           = lookup({name => 'domain_adminserver', default_value => 'AdminServer'}),
  String[1]             $adminserver_address        = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer               $adminserver_port           = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Integer               $nodemanager_port           = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  String[1]             $jrf_target_name            = undef,
  String[1]             $opss_datasource_name       = undef,
  String[1]             $weblogic_user              = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]             $weblogic_password          = lookup({name => 'domain_wls_password'}),
  String[1]             $os_user                    = lookup({name => 'wls_os_user'}),
  String[1]             $os_group                   = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir               = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput                  = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum['on_failure']
    ]
  ]                     $log_output                 = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }
  $domain_dir = "${domains_dir}/${domain_name}"

  unless ( wls_install::jrf_cluster_configured($domain_dir, $jrf_target_name) ) {
    #shutdown adminserver for offline WLST scripts
    wls_install::control{"ShutdownAdminServerForJRF${title}":
      weblogic_home_dir   => $weblogic_home_dir,
      jdk_home_dir        => $jdk_home_dir,
      domain_name         => $domain_name,
      wls_domains_dir     => $domains_dir,
      server_type         => 'admin',
      target              => 'Server',
      server              => $adminserver_name,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
      nodemanager_port    => $nodemanager_port,
      action              => 'stop',
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password,
      os_user             => $os_user,
      os_group            => $os_group,
      download_dir        => $download_dir,
      log_output          => $logoutput,
    }

    file { "${download_dir}/${title}_assignJrfToCluster.py":
      ensure  => present,
      content => template('wls_install/wlst/wlstexec/fmw/assignJrfToCluster.py.erb'),
      backup  => false,
      replace => true,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
    }

    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

    # reorder all apps,libraries, startup , shutdown
    exec { "execwlst assignJrfToCluster.py ${title}":
      command     => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/${title}_assignJrfToCluster.py",
      environment => [
        "JAVA_HOME=${jdk_home_dir}"
      ],
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => [
        File["${download_dir}/${title}_assignJrfToCluster.py"],
        Wls_install::Control["ShutdownAdminServerForJRF${title}"]
      ],
    }

    #startup adminserver for offline WLST scripts
    wls_install::control{"StartupAdminServerForJSF${title}":
      weblogic_home_dir   => $weblogic_home_dir,
      jdk_home_dir        => $jdk_home_dir,
      domain_name         => $domain_name,
      wls_domains_dir     => $domains_dir,
      server_type         => 'admin',
      target              => 'Server',
      server              => $adminserver_name,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
      nodemanager_port    => $nodemanager_port,
      action              => 'start',
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password,
      os_user             => $os_user,
      os_group            => $os_group,
      download_dir        => $download_dir,
      log_output          => $logoutput,
      require             => Exec["execwlst assignJrfToCluster.py ${title}"],
    }
  }
}
