#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::utils::forms11gpatch
#
# installs FMW 11g forms patch
#
##
define wls_install::utils::forms11gpatch(
  Wls_install::Versions $version              = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath  $weblogic_home_dir    = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir  = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $oracle_base_home_dir = lookup({name => 'wls_oracle_base_home_dir'}),
  Stdlib::Absolutepath  $oracle_home_dir      = undef,
  Stdlib::Absolutepath  $jdk_home_dir         = lookup({name => 'wls_jdk_home_dir'}),
  String[1]             $fmw_file1            = undef,
  String[1]             $os_user              = lookup({name => 'wls_os_user'}),
  String[1]             $os_group             = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir         = lookup({name => 'wls_download_dir'}),
  String[1]             $source               = lookup({name => 'wls_source', default_value => undef}),
  Stdlib::Absolutepath  $temp_directory       = lookup({name => 'wls_temp_dir', default_value => '/tmp'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput            = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                     $log_output           = undef, # DEPRECATED
  Boolean               $remote_file          = true, # DEPRECATED
)
{
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  $fmw_product = 'forms_patch'

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"
  $oraInventory = "${oracle_base_home_dir}/oraInventory"

  case $::kernel {
    'Linux': {
      $oraInstPath = '/etc'
      case $::architecture {
        'i386': {
          $installDir = 'linux'
        }
        default: {
          $installDir = 'linux64'
        }
      }
    }
    'SunOS': {
      $oraInstPath = '/var/opt/oracle'
      case $::architecture {
        'i86pc': {
          $installDir = 'intelsolaris'
        }
        default: {
          $installDir = 'solaris'
        }
      }
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }

  }

  $fmw_silent_response_file = 'wls_install/fmw_silent_forms_patch.rsp.erb'
  if ( $oracle_home_dir ) {
    $oracleHome = $oracle_home_dir
  } else {
    $oracleHome = "${middleware_home_dir}/Oracle_FRM1"
  }

  $createFile1 = "${download_dir}/${fmw_product}/Disk1"

  if ( $source ) {
    $mountPoint = $source
  } else {
    $mountPoint = 'puppet:///modules/wls_install/'
  }

  file { "${download_dir}/${title}_silent_${fmw_product}.rsp":
    ensure  => present,
    content => template($fmw_silent_response_file),
    mode    => '0775',
    owner   => $os_user,
    group   => $os_group,
    backup  => false,
  }

  file { "${download_dir}/${fmw_product}":
    ensure => directory,
    mode   => '0775',
    owner  => $os_user,
    group  => $os_group,
    backup => false,
    before => Archive["${download_dir}/${fmw_file1}"],
  }

  archive { "${download_dir}/${fmw_file1}":
    ensure       => present,
    cleanup      => true,
    creates      => $createFile1,
    group        => $os_group,
    extract      => true,
    extract_path => "${download_dir}/${fmw_product}",
    require      => File["${download_dir}/${fmw_product}"],
    source       => "${mountPoint}/${fmw_file1}",
    temp_dir     => $temp_directory,
    user         => $os_user,
  }

  $command = "-silent -response ${download_dir}/${title}_silent_${fmw_product}.rsp -waitforcompletion"

  exec { "install ${fmw_product} ${title}":
    command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/${fmw_product}/Disk1/install/${installDir}/runInstaller ${command} -invPtrLoc ${oraInstPath}/oraInst.loc -ignoreSysPrereqs -jreLoc ${jdk_home_dir} -Djava.io.tmpdir=${temp_directory}'",
    environment => "TEMP=${temp_directory}",
    timeout     => 0,
    path        => $exec_path,
    user        => $os_user,
    group       => $os_group,
    logoutput   => $log_output,
    require     => [
      File["${download_dir}/${title}_silent_${fmw_product}.rsp"],
      Exec["extract ${fmw_file1}"]
    ],
  }
}
