#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::utils::oimconfig
#
# does all the Oracle Identity Management configuration
#
define wls_install::utils::oimconfig(
  Wls_install::Versions $version                 = undef,
  Stdlib::Absolutepath  $oim_home                = undef,
  Boolean               $server_config           = false,
  String[1]             $oim_password            = undef,
  Boolean               $remote_config           = false,
  String[1]             $keystore_password       = undef,
  Boolean               $design_config           = false,
  String[1]             $oimserver_hostname      = undef,
  Integer               $oimserver_port          = 14000,
  String[1]             $soaserver_name          = 'soa_server1',
  String[1]             $oimserver_name          = 'oim_server1',
  Boolean               $bi_enabled              = false,
  Optional[String[1]]   $bi_cluster_name         = undef,
  Optional[String[1]]   $repository_database_url = lookup({name => 'repository_database_url', default_value => undef}),
  String[1]             $repository_prefix       = lookup({name => 'repository_prefix', default_value => 'DEV'}),
  String[1]             $repository_password     = lookup({name => 'repository_password', default_value => 'Welcome01'}),
  Stdlib::Absolutepath  $jdk_home_dir            = lookup({name => 'wls_jdk_home_dir'}),
  Stdlib::Absolutepath  $weblogic_home_dir       = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir     = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $wls_domains_dir         = lookup({name => 'wls_domains_dir', default_value => undef}),
  String[1]             $domain_name             = lookup({name => 'domain_name'}),
  String[1]             $adminserver_name        = lookup({name => 'domain_adminserver', default_value => 'AdminServer'}),
  String[1]             $adminserver_address     = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer               $adminserver_port        = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Integer               $nodemanager_port        = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  String[1]             $weblogic_user           = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]             $weblogic_password       = lookup({name => 'domain_wls_password'}),
  String[1]             $os_user                 = lookup({name => 'wls_os_user'}),
  String[1]             $os_group                = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir            = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput            = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                     $log_output           = undef, # DEPRECATED
) {
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir = $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  $execPath = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  if ( $remote_config ) {
    file { "${download_dir}/${title}config_oim_remote.rsp":
      ensure  => present,
      content => template('wls_install/oim/oim_remote.rsp.erb'),
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
      backup  => false,
    }

    exec { "config oim remote ${title}":
      command   => "${oim_home}/bin/config.sh -silent -response ${download_dir}/${title}config_oim_remote.rsp -waitforcompletion",
      timeout   => 0,
      require   => File["${download_dir}/${title}config_oim_remote.rsp"],
      path      => $execPath,
      user      => $os_user,
      group     => $os_group,
      logoutput => $logoutput,
    }
  }

  if ( $design_config ) {
    file { "${download_dir}/${title}config_oim_design.rsp":
      ensure  => present,
      content => template('wls_install/oim/oim_design.rsp.erb'),
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
      backup  => false,
    }

    exec { "config oim design ${title}":
      command   => "${oim_home}/bin/config.sh -silent -response ${download_dir}/${title}config_oim_design.rsp -waitforcompletion",
      timeout   => 0,
      require   => File["${download_dir}/${title}config_oim_design.rsp"],
      path      => $execPath,
      user      => $os_user,
      group     => $os_group,
      logoutput => $logoutput,
    }
  }

  if ( $server_config ) {

    # check if oim is already configured in this weblogic domain
    $oimValue = wls_install::oim_configured( $domain_dir )

    unless ( $oimValue ) {
      file { "${download_dir}/${title}config_oim_server.rsp":
        ensure  => present,
        content => template('wls_install/oim/oim_server.rsp.erb'),
        mode    => '0775',
        owner   => $os_user,
        group   => $os_group,
        backup  => false,
      }
      exec { "config oim server ${title}":
        command   => "/bin/sh -c 'unset DISPLAY;${oim_home}/bin/config.sh -silent -response ${download_dir}/${title}config_oim_server.rsp -waitforcompletion -debug -logLevel fine'",
        timeout   => 0,
        require   => File["${download_dir}/${title}config_oim_server.rsp"],
        path      => $execPath,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
      }

      if ( $bi_enabled ) {
        # the py script used by the wlst
        file { "${download_dir}/bi-createUDD${title}.py":
          ensure  => present,
          content => template('wls_install/wlst/wlstexec/fmw/bi-createUDD.py.erb'),
          backup  => false,
          replace => true,
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }

        case $::kernel {
          'Linux': {
            $java_statement = 'java'
          }
          'SunOS': {
            $java_statement = 'java -d64'
          }
          default: {
            fail("Unrecognized operating system ${::kernel}")
          }
        }

        $javaCommand = "${java_statement} -Dweblogic.security.SSL.ignoreHostnameVerification=true weblogic.WLST -skipWLSModuleScanning "
        # execute WLST script
        exec { "execwlst bi-createUDD.py ${title}":
          command     => "${javaCommand} ${download_dir}/bi-createUDD${title}.py ${weblogic_password}",
          environment => [
            "CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar",
            "JAVA_HOME=${jdk_home_dir}"
          ],
          path        => $execPath,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [
            File["${download_dir}/bi-createUDD${title}.py"],
            Exec["config oim server ${title}"]
          ],
          before      => Wls_install::Control['stopOIMOimServer1AfterConfig']
        }
      }

      wls_install::control{'stopOIMOimServer1AfterConfig':
        weblogic_home_dir   => $weblogic_home_dir,
        jdk_home_dir        => $jdk_home_dir,
        wls_domains_dir     => $domains_dir,
        domain_name         => $domain_name,
        server_type         => 'managed',
        target              => 'Server',
        server              => $oimserver_name,
        adminserver_address => $adminserver_address,
        adminserver_port    => $adminserver_port,
        nodemanager_port    => $nodemanager_port,
        action              => 'stop',
        weblogic_user       => $weblogic_user,
        weblogic_password   => $weblogic_password,
        os_user             => $os_user,
        os_group            => $os_group,
        download_dir        => $download_dir,
        logoutput           => $logoutput,
        require             => Exec["config oim server ${title}"],
      }

      wls_install::control{'stopOIMSoaServer1AfterConfig':
        weblogic_home_dir   => $weblogic_home_dir,
        jdk_home_dir        => $jdk_home_dir,
        wls_domains_dir     => $domains_dir,
        domain_name         => $domain_name,
        server_type         => 'managed',
        target              => 'Server',
        server              => $soaserver_name,
        adminserver_address => $adminserver_address,
        adminserver_port    => $adminserver_port,
        nodemanager_port    => $nodemanager_port,
        action              => 'stop',
        weblogic_user       => $weblogic_user,
        weblogic_password   => $weblogic_password,
        os_user             => $os_user,
        os_group            => $os_group,
        download_dir        => $download_dir,
        logoutput           => $logoutput,
        require             => [
          Wls_install::Control['stopOIMOimServer1AfterConfig'],
          Exec["config oim server ${title}"]
        ],
      }

      wls_install::control{'stopOIMAdminServerAfterConfig':
        weblogic_home_dir   => $weblogic_home_dir,
        jdk_home_dir        => $jdk_home_dir,
        wls_domains_dir     => $domains_dir,
        domain_name         => $domain_name,
        server_type         => 'admin',
        target              => 'Server',
        server              => $adminserver_name,
        adminserver_address => $adminserver_address,
        adminserver_port    => $adminserver_port,
        nodemanager_port    => $nodemanager_port,
        action              => 'stop',
        weblogic_user       => $weblogic_user,
        weblogic_password   => $weblogic_password,
        os_user             => $os_user,
        os_group            => $os_group,
        download_dir        => $download_dir,
        logoutput           => $logoutput,
        require             => [
          Wls_install::Control['stopOIMOimServer1AfterConfig'],
          Wls_install::Control['stopOIMSoaServer1AfterConfig'],
          Exec["config oim server ${title}"]
        ],
      }

      wls_install::control{'startOIMAdminServerAfterConfig':
        weblogic_home_dir   => $weblogic_home_dir,
        jdk_home_dir        => $jdk_home_dir,
        wls_domains_dir     => $domains_dir,
        domain_name         => $domain_name,
        server_type         => 'admin',
        target              => 'Server',
        server              => $adminserver_name,
        adminserver_address => $adminserver_address,
        adminserver_port    => $adminserver_port,
        nodemanager_port    => $nodemanager_port,
        action              => 'start',
        weblogic_user       => $weblogic_user,
        weblogic_password   => $weblogic_password,
        os_user             => $os_user,
        os_group            => $os_group,
        download_dir        => $download_dir,
        logoutput           => $logoutput,
        require             => [
          Wls_install::Control['stopOIMOimServer1AfterConfig'],
          Wls_install::Control['stopOIMAdminServerAfterConfig'],
          Wls_install::Control['stopOIMSoaServer1AfterConfig'],
          Exec["config oim server ${title}"]
        ],
      }
    }
  }
}
