#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: wls_install::utils::rcu
#    rcu for adf 12.1.2 & 12.1.3
#
define wls_install::utils::rcu(
  Wls_install::Versions   $version                     = lookup({name => 'wls_version', default_value => 1111}),
  Enum['adf','soa','mft'] $fmw_product                 = 'adf',
  Stdlib::Absolutepath    $oracle_fmw_product_home_dir = undef,
  Stdlib::Absolutepath    $jdk_home_dir                = lookup({name => 'wls_jdk_home_dir'}),
  String[1]               $os_user                     = lookup({name => 'wls_os_user'}),
  String[1]               $os_group                    = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath    $download_dir                = lookup({name => 'wls_download_dir'}),
  Enum['create','delete'] $rcu_action                  = 'create',
  Optional[String[1]]     $rcu_jdbc_url                = undef,
  Optional[String[1]]     $rcu_database_url            = undef,
  Optional[String[1]]     $rcu_prefix                  = undef,
  Optional[String[1]]     $rcu_password                = undef,
  String[1]               $rcu_sys_user                = 'sys',
  Optional[String[1]]     $rcu_sys_password            = undef,
  Optional[Boolean]       $rcu_honor_omf               = undef,
  Optional[
    Enum[
      'SMALL',
      'MED',
      'LARGE'
    ]
  ]                       $rcu_soa_profile             = undef,
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                       $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum['on_failure']
    ]
  ]                       $log_output                  = undef, # DEPRECATED
){
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  case $::kernel {
    'Linux', 'SunOS': {
      $execPath = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'
    }
    default: {
      fail('Unrecognized operating system')
    }
  }

  case $rcu_honor_omf {
    true: {
      $honor_omf = '-honorOMF true'
    }
    false: {
      $honor_omf = '-honorOMF false'
    }
    undef: {
      $honor_omf = ''
    }
    default: {
      fail("Unrecognized rcu_honor_omf value ${rcu_honor_omf}. Supported values are undef, true or false")
    }
  }

  case $rcu_soa_profile {
    'SMALL', 'MED', 'LARGE': {
      $soa_profile = "-variables SOA_PROFILE_TYPE=${rcu_soa_profile}"
    }
    undef: {
      $soa_profile = ''
    }
    default: {
      fail("Unrecognized rcu_soa_profile value ${rcu_soa_profile}. Supported values are undef, SMALL, MED & LARGE")
    }
  }

  case $fmw_product {
    'adf': {
      case $version {
        1036, 1111, 1211, 1212, 1213: {
          $components = '-component MDS -component IAU -component IAU_APPEND -component IAU_VIEWER -component OPSS -component WLS -component UCSCC '
          $componentsPasswords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
        default: {
          $components = '-component MDS -component IAU -component IAU_APPEND -component IAU_VIEWER -component OPSS -component WLS -component STB '
          $componentsPasswords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
      }
    }
    'soa': {
      case $version {
        1036, 1111, 1211, 1212, 1213: {
          $components = '-component MDS -component IAU -component IAU_APPEND -component IAU_VIEWER -component OPSS -component WLS -component UCSCC -component UCSUMS -component UMS -component ESS -component SOAINFRA -component MFT '
          $componentsPasswords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
        default: {
          $components = '-component MDS -component IAU -component IAU_APPEND -component IAU_VIEWER -component OPSS -component WLS -component STB -component UCSUMS -component ESS -component SOAINFRA '
          $componentsPasswords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
      }
    }
    'mft': {
      $components = '-component MDS -component IAU -component IAU_APPEND -component IAU_VIEWER -component OPSS -component WLS -component UCSCC -component MFT -component UCSUMS -component ESS'
      $componentsPasswords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
    }
    default: {
      fail("Unrecognized FMW fmw_product ${fmw_product}")
    }
  }

  file { "${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt":
    ensure  => present,
    content => template('wls_install/utils/rcu_passwords.txt.erb'),
    mode    => '0600',
    owner   => $os_user,
    group   => $os_group,
    backup  => false,
    before  => Wls_rcu[$rcu_prefix],
  }

  unless ( defined(File["${download_dir}/checkrcu.py"]) ) {
    file { "${download_dir}/checkrcu.py":
      ensure => present,
      source => 'puppet:///modules/wls_install/wlst/checkrcu.py',
      mode   => '0775',
      owner  => $os_user,
      group  => $os_group,
      backup => false,
      before => Wls_rcu[$rcu_prefix],
    }
  }

  if ( $rcu_action == 'create' ) {
    $action = '-createRepository'
    $omf = $honor_omf
  } else {
    $action = '-dropRepository'
    $omf = ''
  }

  wls_rcu{ $rcu_prefix:
    ensure       => $rcu_action,
    statement    => "${oracle_fmw_product_home_dir}/bin/rcu -silent ${action} -databaseType ORACLE -connectString ${rcu_database_url} -dbUser ${rcu_sys_user} -dbRole SYSDBA ${soa_profile} -schemaPrefix ${rcu_prefix} ${components} ${omf} -f < ${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt",
    os_user      => $os_user,
    oracle_home  => $oracle_fmw_product_home_dir,
    sys_user     => $rcu_sys_user,
    sys_password => $rcu_sys_password,
    jdbc_url     => $rcu_jdbc_url,
    jdk_home_dir => $jdk_home_dir,
    check_script => "${download_dir}/checkrcu.py",
  }

  -> exec{'remove_pwd_file':
    command => "/bin/rm ${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt",
    onlyif  => "/bin/test -f ${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt",
  }
}
