#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: wls_install::utils::webtier
#
# Add a Webtier to the Enterprise manager
##
define wls_install::utils::webtier(
  Wls_install::Versions   $version                    = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath    $weblogic_home_dir          = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath    $middleware_home_dir        = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath    $jdk_home_dir               = lookup({name => 'wls_jdk_home_dir'}),
  Stdlib::Absolutepath    $wls_domains_dir            = lookup({name => 'wls_domains_dir', default_value => undef}),
  String[1]               $domain_name                = lookup({name => 'domain_name'}),
  String[1]               $adminserver_address        = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer                 $adminserver_port           = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Enum['create','delete'] $action_name                = 'create',
  Boolean                 $webgate_configure          = false,
  Optional[String[1]]     $webgate_agentname          = lookup({name => 'webgate_agentname', default_value => undef}),
  Optional[String[1]]     $webgate_hostidentifier     = lookup({name => 'webgate_hostidentifier', default_value => undef}),
  String[1]               $oamadminserverhostname     = lookup({name => 'oamadminserverhostname', default_value => 'localhost'}),
  Integer                 $oamadminserverport         = lookup({name => 'oamadminserverport', default_value => 7001}),
  Boolean                 $domain_configure           = true,
  Optional[String[1]]     $instance_name              = undef,
  Optional[String[1]]     $machine_name               = undef,
  String[1]               $weblogic_user              = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  String[1]               $weblogic_password          = lookup({name => 'domain_wls_password'}),
  String[1]               $os_user                    = lookup({name => 'wls_os_user'}),
  String[1]               $os_group                   = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath    $download_dir               = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                       $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum['on_failure']
    ]
  ]                       $log_output                  = undef, # DEPRECATED
){
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }
  $domain_dir = "${domains_dir}/${domain_name}"


  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  if ( $version == 1212 or $version == 1213 or $version >= 1221 ){
    file { "${download_dir}/${title}_createWebtier.py":
      ensure  => present,
      content => template('wls_install/wlst/wlstexec/fmw/createWebtier.py.erb'),
      backup  => false,
      replace => true,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
    }

    if ( $action_name == 'create' ) {
      exec { "execwlst webtier ${title}":
        command     => "${middleware_home_dir}/ohs/common/bin/wlst.sh ${download_dir}/${title}_createWebtier.py ${weblogic_password}",
        environment => "JAVA_HOME=${jdk_home_dir}",
        path        => $exec_path,
        creates     => "${domain_dir}/system_components/OHS/${instance_name}",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => File["${download_dir}/${title}_createWebtier.py"],
      }
    } else {
      exec { "execwlst webtier ${title}":
        command     => "${middleware_home_dir}/ohs/common/bin/wlst.sh ${download_dir}/${title}_createWebtier.py ${weblogic_password}",
        environment => "JAVA_HOME=${jdk_home_dir}",
        path        => $exec_path,
        onlyif      => "test -d ${domain_dir}/system_components/OHS/${instance_name}",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => File["${download_dir}/${title}_createWebtier.py"],
      }
    }
  } else {

    $instance_home = "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}"
    $instance_id = "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/config/OHS/${instance_name}"

    file { "${download_dir}/${title}_configureWebtier.rsp":
      ensure  => present,
      content => template('wls_install/wlst/wlstexec/fmw/configureWebtier.rsp.erb'),
      backup  => false,
      replace => true,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
    }

    exec { "config webtier ${title}":
      command     => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/bin/config.sh -silent -response ${download_dir}/${title}_configureWebtier.rsp -waitforcompletion'",
      environment => "JAVA_HOME=${jdk_home_dir}",
      path        => $exec_path,
      creates     => "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}",
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => File["${download_dir}/${title}_configureWebtier.rsp"],
    }

    if ( $webgate_configure ) {

      # Note: rreg really should NOT be run from the "WebGate" machine
      # instead it should be run on the OAM server, the output artifacts stored in a secure place,
      # and then copied from there upon installation of the WebGate
      # for now this is an OK start

      # first create the input file:
      file { "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/input/${webgate_agentname}.xml":
        ensure  => present,
        content => template('wls_install/oam/rreg_input.xml.erb'),
        backup  => false,
        replace => true,
        mode    => '0440',
        owner   => $os_user,
        group   => $os_group,
      }

      file { "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh":
        ensure => file,
        mode   => '0700',
      }

      # in the real world rreg wouldn't be run on the host running the WebTier bits
      exec { "Run rreg for WebGate instance ${webgate_agentname}":
        command     => "/bin/echo -e ${weblogic_user}\\\\n${weblogic_password}\\\\n|${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh inband input/${webgate_agentname}.xml -noprompt",
        environment => "JAVA_HOME=${jdk_home_dir}",
        cwd         => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg",
        creates     => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/output/${webgate_agentname}",
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          File["${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/input/${webgate_agentname}.xml"],
          File["${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh"]
        ],
      }

      exec { "Deploy webgate to ${instance_id}":
        command     => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/deployWebGate/deployWebGateInstance.sh -w ${instance_id} -oh ${middleware_home_dir}/Oracle_OAMWebGate1",
        environment => "LD_LIBRARY_PATH='${middleware_home_dir}/Oracle_WT1/lib'",
        cwd         => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/deployWebGate",
        creates     => "${instance_id}/webgate",
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["config webtier ${title}"],
          Exec["Run rreg for WebGate instance ${webgate_agentname}"]
        ],
      }

      # copy the output from rreg to the WebGate
      # in the real world the rreg artifacts would:
      # * be copied to this host during host provisioning
      #   OR
      # * would be stored on a fileserver accessible from this host
      #   e.g. from puppet:
      file { "Copy rreg artifacts for ${instance_id}":
        path    => "${instance_id}/webgate/config",
        source  => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/output/${webgate_agentname}",
        recurse => true,
        require => [
          Exec["Deploy webgate to ${instance_id}"],
          Exec["Run rreg for WebGate instance ${webgate_agentname}"]
        ],
      }

      exec { "EditHttpConf to enable webgate ${title}":
        command     => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/setup/InstallTools/EditHttpConf -w ${instance_id} -oh ${middleware_home_dir}/Oracle_OAMWebGate1",
        environment => [
          "LD_LIBRARY_PATH=${middleware_home_dir}/Oracle_WT1/lib",
          "ORACLE_HOME=${middleware_home_dir}/Oracle_OAMWebGate1"
        ],
        cwd         => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/setup/InstallTools",
        path        => $exec_path,
        unless      => "grep -c '${instance_id}/webgate.conf' ${instance_id}/httpd.conf",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["Deploy webgate to ${instance_id}"],
          File["Copy rreg artifacts for ${instance_id}"]
        ],
      }

      # after enabling a WebGate you need to bounce OHS
      exec { "Stop OHS instance ${title}":
        command   => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/bin/opmnctl stopall'",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => Exec["EditHttpConf to enable webgate ${title}"],
      }

      exec { "Start OHS instance ${title}":
        command   => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/bin/opmnctl startall'",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => Exec["Stop OHS instance ${title}"],
      }
    }
  }
}
